package teen

import (
	"server/game/room"
	"server/msg"

	"github.com/name5566/leaf/log"
)

func buildRoom(teenPattiRoom *room.Room) *msg.ReqRoom {
	return &msg.ReqRoom{
		Id:         teenPattiRoom.Id,
		Status:     int32(teenPattiRoom.Status),
		Round:      int32(teenPattiRoom.Round),
		ReqPlayers: convertToMsgPlayerList(teenPattiRoom.Players),
		GameRound:  convertToMsgGameRound(teenPattiRoom.GameRound),
	}
}

func convertToMsgPlayerList(players []*room.Player) []*msg.ReqPlayer {
	msgPlayers := make([]*msg.ReqPlayer, len(players))
	for i, player := range players {
		msgPlayers[i] = &msg.ReqPlayer{
			Id: player.Id,
			// HandCards: convertToMsgCardList(player.HandCards),
			SitPos:   &player.SitPos,
			IsDealer: &player.IsDealer,
			IsSeen:   &player.IsSeen,
			IsPacked: &player.IsPacked,
			IsShow:   &player.IsShow,
		}
	}
	return msgPlayers
}

func convertToMsgCardList(cards *[]msg.ReqCard) []*msg.ReqCard {
	msgCards := make([]*msg.ReqCard, len(*cards))
	for i, card := range *cards {
		msgCards[i] = &msg.ReqCard{
			Color: int32(card.Color),
			Point: int32(card.Point),
		}
	}
	return msgCards
}

func convertToMsgGameRound(gameRound *room.GameRound) *msg.ReqGameRound {
	log.Debug("convertToMsgGameRound: %v", gameRound)
	return &msg.ReqGameRound{
		Rounds: convertToMsgRoundList(gameRound.Rounds),
	}
}
func convertToMsgRoundList(rounds []msg.ReqRound) []*msg.ReqRound {
	msgRounds := make([]*msg.ReqRound, len(rounds))
	for i := range rounds {
		msgRounds[i] = &rounds[i]
	}
	return msgRounds
}

func convertToMsgRoomList(rooms []TeenPattiRoom) *msg.TeenPattiRoomList {
	msgRooms := make([]*msg.TeenPattiRoom, len(rooms))
	for i, room := range rooms {
		msgRooms[i] = &msg.TeenPattiRoom{
			Boot:         room.Boot,
			MinBuyin:     room.MinBuyin,
			ChaalLimmit:  room.ChaalLimmit,
			PotLimit:     room.PotLimit,
			TotalPlayers: room.TotalPlayers,
			RoomLevel:    room.RoomLevel,
			RoomId:       room.RoomId,
			Type:         room.Type,
		}
	}
	return &msg.TeenPattiRoomList{
		TeenPattiRoom: msgRooms,
	}
}
