package teen

import (
	"fmt"
	"server/game"
	"server/game/room"
	"server/msg"
	"time"

	"github.com/name5566/leaf/log"
)

// 看牌
func seen(room *room.Room, sitPos int32) {
	log.Debug("看牌的位置: %d", room.RoundSitPos)
	player := room.GetPlayerBySitPos(sitPos)
	player.IsSeen = true
	seenCards := *room.GetPlayerBySitPos(sitPos).HandCards
	SendRoundMsgToAll(room, &msg.ReqRound{
		Round:       int32(room.Round),
		RoundSitPos: int32(sitPos),
		PlayerOpt:   &msg.PlayerOpt{OptType: msg.PlayerOptType_OPT_SEEN, SeenCards: convertToMsgCardList(&seenCards)},
		UserId:      room.GetPlayerBySitPos(sitPos).GetUserId(),
	})
}

// 弃牌
func packed(room *room.Room, sitPos int32) {
	player := room.GetPlayerBySitPos(sitPos)
	player.IsPacked = true
	if isOnlyOne, sp := room.IsOnlyOneNotPacked(); isOnlyOne {
		// 如果所有玩家都弃牌，判定当前座位玩家获胜
		room.PlayerWin(sp)
	}
	room.SetNextRound()
	log.Debug("弃牌的位置: %d", room.RoundSitPos)
	SendRoundMsgToAll(room, &msg.ReqRound{
		Round:       int32(room.Round),
		RoundSitPos: int32(sitPos),
		PlayerOpt:   &msg.PlayerOpt{OptType: msg.PlayerOptType_OPT_PACKED},
		UserId:      room.GetPlayerBySitPos(sitPos).GetUserId(),
	})
	checkRoomStatus(room)
}

// 跟注
func chaal(room *room.Room, sitPos int32) {
	room.SetNextRound()
	SendRoundMsgToAll(room, &msg.ReqRound{
		Round:       int32(room.Round),
		RoundSitPos: int32(sitPos),
		PlayerOpt:   &msg.PlayerOpt{OptType: msg.PlayerOptType_OPT_CHAAL},
		UserId:      room.GetPlayerBySitPos(sitPos).GetUserId(),
	})
	checkRoomStatus(room)
}

// 亮牌
func show(room *room.Room, sitPos int32) {
	room.SetNextRound()
	SendRoundMsgToAll(room, &msg.ReqRound{
		Round:       int32(room.Round),
		RoundSitPos: int32(sitPos),
		PlayerOpt:   &msg.PlayerOpt{OptType: msg.PlayerOptType_OPT_SHOW},
		UserId:      room.GetPlayerBySitPos(sitPos).GetUserId(),
	})
}

// 取消操作超时
func cancelOptTimeout(player *room.Player, sitPos int32) {
	eventID := fmt.Sprintf("player_%s_opt_%s", player.Id, sitPos)
	timerMgr.RemoveEvent(eventID)
}

// 房间状态检测
func checkRoomStatus(r *room.Room) {
	log.Debug("下一个操作的玩家座位：%d", r.RoundSitPos)
	game.Module.Skeleton.AfterFunc(time.Second*1, func() {
		notifyPlayerAction(r, int32(r.RoundSitPos), msg.PlayerOptType_OPT_SELECT)
	})
}
